import QtQuick 1.1
import Widgets 0.1
import "Dash"
import "Applications"
import "Greeter"
import "Spread"
import "Launcher"
import "Indicators"
import "Notifications"
import "OnScreenKeyboard"
import "Hud"
import "SideStage"
import "Widgets"
import "Math.js" as MathLocal

Item {
    id: shell

    width: tablet ? units.dp(1280) : units.dp(320)
    height: tablet ? units.dp(800) : units.dp(854/1.5)

    property bool tablet: true


    property ListModel phoneApplicationsModel: ApplicationsModel {}
    property ListModel tabletApplicationsModel: TabletApplicationsModel {}
    property ListModel applicationsModel: tablet ? tabletApplicationsModel : phoneApplicationsModel

    property ListModel sideStageApplicationsModel: sideStageFilter.filteredModel
    property variant sideStageFilter: FilterModel {
        sourceModel: applicationsModel
        function isItemAccepted(item) {
            return item.sideStage
        }
    }

    property ListModel mainStageApplicationsModel: mainStageFilter.filteredModel
    property variant mainStageFilter: FilterModel {
        sourceModel: applicationsModel
        function isItemAccepted(item) {
            return !item.sideStage
        }
    }

    Keys.onPressed: {
        if (event.key == Qt.Key_1) {
            activateApplication("twitter")
        } else if (event.key == Qt.Key_2) {
            activateApplication("foursquare")
        } else if (event.key == Qt.Key_3) {
            activateApplication("calculator")
        } else if (event.key == Qt.Key_4) {
            activateApplication("gallery")
        } else if (event.key == Qt.Key_5) {
            activateApplication("worldofgoo")
        } else if (event.key == Qt.Key_5) {
            activateApplication("maps")
        } else if (event.key == Qt.Key_5) {
            activateApplication("facebook")
        } else if (event.key == Qt.Key_0) {
            activateDash()
        }
    }

    function activateApplication(desktopFile) {
        applications.activateApplication(desktopFile)
        sideStage.activateApplication(desktopFile)
    }

    function activateDash() {
        dash.show()
    }

    function showOnScreenKeyboard() {
        onScreenKeyboard.show()
    }

    Units {
        id: units
    }

    ImageMipmap {
        id: bg
        resource: tablet ? "graphics/tablet_background.jpg" : "graphics/phone_background.jpg"
        anchors.fill: parent
    }

    Dash {
        id: dash

        available: !greeter.shown
        hides: [applications, launcher, spread, indicators]
        shown: true

        anchors {
            top: parent.top
            topMargin: indicators.topbarHeight
            bottom: parent.bottom
        }
        width: parent.width
        scale: 1 - 0.2*applicationsRevealer.animatedProgress
        spreadModel: applications.runningModel

        opacity: applicationsRevealer.animatedProgress < 0.99 ? 1 : 0
        Behavior on opacity {StandardAnimation{}}

        onApplicationSelected: activateApplication(identifier)
    }

    Applications {
        id: applications

        available: !greeter.shown
        hides: [launcher, spread, indicators]
        shown: false
        onShownChanged: {
            if (!shown) {
                sideStage.hide()
            }
        }

        opacity: 1.0
        showAnimation: StandardAnimation { property: "x"; duration: 350; to: applicationsRevealer.openedValue; easing.type: Easing.OutCubic }
        hideAnimation: StandardAnimation { property: "x"; duration: 350; to: applicationsRevealer.closedValue; easing.type: Easing.OutCubic }

        y: indicators.topbarHeight
        width: parent.width
        height: parent.height - indicators.topbarHeight - indicators.separatorLineHeight
        applicationsModel: shell.mainStageApplicationsModel
    }

    Revealer {
        id: applicationsRevealer

        property real animatedProgress: -dragPosition / closedValue
        Behavior on animatedProgress {enabled: !applicationsRevealer.dragging; NumberAnimation{duration: 350; easing.type: Easing.OutCubic}}

        enabled: applications.runningModel.count > 0
        direction: Qt.RightToLeft
        openedValue: 0
        hintDisplacement: units.dp(20)
        closedValue: width
        target: applications
        width: applications.width
        height: applications.height
        orientation: Qt.Horizontal
    }

    SideStage {
        id: sideStage

        available: !greeter.shown
        openingDirection: Qt.RightToLeft
        shown: false
        opacity: 1.0
        showAnimation: StandardAnimation { property: "x"; duration: 350; to: sideStageRevealer.openedValue; easing.type: Easing.OutCubic }
        hideAnimation: StandardAnimation { property: "x"; duration: 350; to: sideStageRevealer.closedValue; easing.type: Easing.OutCubic }

        visible: tablet
        y: indicators.topbarHeight
        width: tablet ? units.dp(320) + leftMargin : units.dp(0)
        height: parent.height - indicators.topbarHeight - indicators.separatorLineHeight
        applicationsModel: shell.sideStageApplicationsModel
        leftMargin: units.dp(5)
        handleVisible: sideStageRevealer.draggingArea.pressed && shown
    }

    Revealer {
        id: sideStageRevealer

        visible: !(applications.runningModel.count != 0 && !applications.shown)
        direction: Qt.RightToLeft
        openedValue: shell.width - sideStage.width
        closedValue: shell.width
        hintDisplacement: units.dp(20)
        handleSize: units.dp(35)
        target: sideStage
        width: sideStage.width + units.dp(12)
        height: sideStage.height
        anchors.top: sideStage.top
        orientation: Qt.Horizontal
        anchors.right: parent.right

        function dragToValue(dragPosition) {
            var offset = target.shown ? hintDisplacement : 0
            return dragPosition + closedValue + offset
        }
    }

    Greeter {
        id: greeter

        available: true
        hides: [launcher, spread, indicators]
        shown: true
        showAnimation: StandardAnimation { property: "x"; to: greeterRevealer.openedValue }
        hideAnimation: StandardAnimation { property: "x"; to: greeterRevealer.closedValue }

        y: indicators.topbarHeight
        width: parent.width
        height: parent.height - indicators.topbarHeight
    }

    Revealer {
        id: greeterRevealer

        target: greeter
        width: greeter.width
        height: greeter.height
        orientation: Qt.Horizontal
        visible: greeter.shown
    }

    Item {
        id: overlay

        anchors.fill: parent

        Spread {
            id: spread

            available: !indicators.shown && !greeter.shown
            hides: [indicators]
            shown: false
            showAnimation: StandardAnimation { property: "x"; to: spread.openedValue }
            hideAnimation: StandardAnimation { property: "x"; to: spread.closedValue }

            anchors {
                top: parent.top
                bottom: parent.bottom
            }
            width: parent.width - launcher.width

            property real openedValue: launcherRevealer.openedValue + launcher.width
            property real closedValue: -spread.width
            property real dragAmount: available ? launcherRevealer.dragPosition : 0
            property real swipedThreshold: 0

            Binding {
                target: spread
                property: "x"
                value: MathLocal.clamp(spread.closedValue + spread.dragAmount, spread.closedValue, spread.openedValue)
                when: launcherRevealer.dragging
            }
            Component.onCompleted: x = shown ? openedValue : closedValue

            Connections {
                target: launcherRevealer
                onDraggingChanged: {
                    if (launcherRevealer.dragging) return
                    if (Math.abs(launcherRevealer.draggingArea.dragVelocity) >= launcherRevealer.dragVelocityThreshold
                        && launcherRevealer.draggingArea.dragVelocity > 0
                        && spread.dragAmount > spread.swipedThreshold) {
                        spread.show()
                    } else {
                        spread.hide()
                    }
                }
            }
        }

        Launcher {
            id: launcher

            available: !indicators.shown
            hides: [indicators]
            shown: false
            showAnimation: StandardAnimation { property: "x"; to: launcherRevealer.openedValue }
            hideAnimation: StandardAnimation { property: "x"; to: launcherRevealer.closedValue }

            width: units.dp(134/1.5)
            height: parent.height
        }

        Revealer {
            id: launcherRevealer

            target: launcher
            width: launcher.width
            height: launcher.height
            orientation: Qt.Horizontal
            hintDisplacement: units.dp(40/1.5)
            visible: false
        }

        Indicators {
            id: indicators

            width: parent.width
            height: parent.height
            rowWidth: tablet ? units.dp(320) : parent.width
            available: !greeter.shown
            hides: [launcher, spread]
            shown: false
            revealer: indicatorRevealer
            hintValue: indicatorRevealer.hintDisplacement
            topbarHeight: units.dp(24)
            showAnimation: StandardAnimation { property: "progress"; duration: 350; to: indicatorRevealer.openedValue; easing.type: Easing.OutCubic }
            hideAnimation: StandardAnimation { property: "progress"; duration: 350; to: indicatorRevealer.closedValue; easing.type: Easing.OutCubic }
            openedHeight: parent.height

            onStateChanged: {
                if (!tablet) {
                    if (state == "reveal" || state == "hint") {
                        hud.hideDown()
                    } else if (state == "initial") {
                        hud.show()
                    } else if (state == "locked" || state == "commit") {
                        hud.hideUp()
                    }
                }
            }
        }

        Revealer {
            id: indicatorRevealer

            width: indicators.rowWidth
            anchors.right: parent.right
            height: parent.height
            target: indicators
            handleSize: indicators.topbarHeight
            closedValue: indicators.topbarHeight + indicators.separatorLineHeight
            openedValue: indicators.openedHeight
            dragVelocityThreshold: units.dp(1/1.5)
            boundProperty: "progress"
            hintDisplacement: 3*indicators.topbarHeight
            orientation: Qt.Vertical

            onOpenClicked: {
                indicators.openOverview()
                indicators.show()
            }
            onCloseClicked: {
                indicators.hide()
            }

            onOpenPressed: indicators.handlePress()

            function dragToValue(dragPosition) {
                var offset = target.shown ? 0 : -hintDisplacement + handleSize
                return dragPosition + offset
            }
        }

        Hud {
            id: hud

            width: units.dp(100)
            height: indicators.topbarHeight
            anchors {
                top: indicators.top
                left: indicators.left
            }
        }

        Notifications {
            id: notifications

            width: parent.width
            height: parent.height
            y: indicators.shown ? 0 : indicators.topbarHeight + indicators.separatorLineHeight
            onNotificationDismissed: {
                indicators.newMessageData(data)
            }
            onActivateApplication: shell.activateApplication(application)
        }
    }

    focus: true
    OnScreenKeyboard {
        id: onScreenKeyboard

        // FIXME: should be visible when a TextInput has the focus
        shown: false
        y: parent.height
        showAnimation: StandardAnimation { property: "y"; to: onScreenKeyboard.parent.height - onScreenKeyboard.height }
        hideAnimation: StandardAnimation { property: "y"; to: onScreenKeyboard.parent.height }

        width: parent.width
    }

    Item {
        id: quitButton
        width: units.dp(100/1.5)
        height: units.dp(50/1.5)
        MouseArea {
            anchors.fill: parent
            onPressAndHold: quitResetDialog.show()
        }
    }

    QuitResetDialog {
        id: quitResetDialog
        anchors.fill: parent
    }
}
